"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _types = require("@babel/types");

var _utils = require("./utils");

var _ref = _interopRequireDefault(require("./labels/ref"));

var _constant = require("./constant");

var _watch = _interopRequireDefault(require("./labels/watch"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function reactLabelSugar(_, options) {
  var _options$refLabel = options.refLabel,
      refLabel = _options$refLabel === void 0 ? _constant.DefaultRefLabel : _options$refLabel,
      _options$watchLabel = options.watchLabel,
      watchLabel = _options$watchLabel === void 0 ? _constant.DefaultWatchLabel : _options$watchLabel,
      _options$ignoreMember = options.ignoreMemberExpr,
      ignoreMemberExpr = _options$ignoreMember === void 0 ? _constant.DefaultIgnoreMemberExpr : _options$ignoreMember;
  return {
    visitor: {
      LabeledStatement: function LabeledStatement(path, state) {
        var node = path.node,
            scope = path.scope;
        var label = node.label,
            body = node.body;

        if (!(0, _types.isExpressionStatement)(body)) {
          throw new Error("ref sugar must be an expression statement");
        }

        switch (label.name) {
          case refLabel:
            {
              (0, _ref["default"])(body.expression, scope, options, function (newBody, ref) {
                var _state$reactRefs;

                path.replaceWith(newBody);
                (_state$reactRefs = state.reactRefs) !== null && _state$reactRefs !== void 0 ? _state$reactRefs : state.reactRefs = [];
                state.reactRefs.push(ref);
              });
              break;
            }

          case watchLabel:
            {
              (0, _watch["default"])(body.expression, scope, options, function (newBody) {
                return path.replaceWith(newBody);
              });
              break;
            }

          default:
            {
              break;
            }
        }
      },
      AssignmentExpression: function AssignmentExpression(path, state) {
        if (!state.reactRefs) return;
        var node = path.node,
            scope = path.scope;
        var ref = (0, _utils.getTargetRef)(node.left, state.reactRefs);
        if (!ref || !(0, _utils.isRefInTargetScope)(scope, ref)) return;

        if ((0, _types.isMemberExpression)(node.left)) {
          !ignoreMemberExpr && (0, _utils.handleMemberExpression)(node.left, ref, path);
        } else {
          var arrowReturn = node.operator === "=" ? node.right : (0, _types.binaryExpression)(node.operator.slice(0, -1), ref.identify, node.right);
          path.replaceWith((0, _types.callExpression)(ref.modifier, [(0, _types.arrowFunctionExpression)([ref.identify], arrowReturn)]));
        }
      },
      UpdateExpression: function UpdateExpression(path, state) {
        if (!state.reactRefs) return;
        var node = path.node,
            scope = path.scope;
        var ref = (0, _utils.getTargetRef)(node.argument, state.reactRefs);
        if (!ref || !(0, _utils.isRefInTargetScope)(scope, ref)) return;

        if ((0, _types.isMemberExpression)(node.argument)) {
          !ignoreMemberExpr && (0, _utils.handleMemberExpression)(node.argument, ref, path);
          return;
        }

        path.replaceWith((0, _types.callExpression)(ref.modifier, [(0, _types.arrowFunctionExpression)([ref.identify], (0, _types.binaryExpression)(node.operator === "++" ? "+" : "-", ref.identify, (0, _types.numericLiteral)(1)))]));
      }
    }
  };
}

var _default = reactLabelSugar;
exports["default"] = _default;