"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.capitalize = capitalize;
exports.getTargetRef = getTargetRef;
exports.handleMemberExpression = handleMemberExpression;
exports.isRefInTargetScope = isRefInTargetScope;

var _types = require("@babel/types");

function capitalize(str) {
  if (str.length === 0) return str;
  return str.charAt(0).toUpperCase() + str.slice(1);
}
/**
 * Get target value from "Identifier" or "MemberExpression"
 * eg: a = 1;          =>  a
 * eg: obj.value = 1;  =>  obj
 */


function getTargetRef(leftValue, refs) {
  if (!(0, _types.isIdentifier)(leftValue) && !(0, _types.isMemberExpression)(leftValue)) return undefined;

  if ((0, _types.isIdentifier)(leftValue)) {
    var target = leftValue.name;
    return refs.find(function (ref) {
      return ref.identify.name === target;
    });
  } else {
    return getTargetRef(leftValue.object, refs);
  }
}
/**
 * move member expression to setState callback;
 * eg: obj.value = 1;  =>  setState(obj => { obj.value = 1 });
 */


function handleMemberExpression(node, ref, path) {
  var _node$extra, _node$extra2;

  if ((_node$extra = node.extra) === null || _node$extra === void 0 ? void 0 : _node$extra.REACT_REF_HAS_SETTED) return;
  path.replaceWith((0, _types.callExpression)(ref.modifier, [(0, _types.arrowFunctionExpression)([ref.identify], (0, _types.blockStatement)([(0, _types.expressionStatement)(path.node)]))]));
  (_node$extra2 = node.extra) !== null && _node$extra2 !== void 0 ? _node$extra2 : node.extra = {};
  node.extra.REACT_REF_HAS_SETTED = true;
}
/**
 * check if ref binding has be override in current scope
 * check if ref is not included in current scope
 */


function isRefInTargetScope(scope, ref) {
  if (!scope) return false;
  if (scope === ref.scope) return true; // if has overwritten ref in the path, return false;

  if (scope.bindings[ref.identify.name]) return false;
  return isRefInTargetScope(scope.parent, ref);
}